﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using SilverMotion.EnvironmentTypes;

namespace SkyBox
{
    public partial class MainPage : UserControl
    {
        RenderBranch skyBox;
        bool inMotion;
        Point mousePos;
        float azimuth = -(float)(Math.PI / 4),
              elevation = (float)(Math.PI / 6);

        public MainPage()
        {
            InitializeComponent();
        }

        private void environment_Loaded(object sender, RoutedEventArgs e)
        {
            skyBox = environment.Hierarchy.Find("SkyBox");

            // Set shading of the skybox so that it renders directly the color of the texture, regardless of any lights
            skyBox.MaterialReference.ClearShaders();
            skyBox.MaterialReference.AddShader(TextureShader.ColorMap, ShaderColorOperation.Set, environment.LoadedResources.FindTexture("skybox.zip/VV9.JPG", false));

            RefreshCamera();
        }

        private void environment_MouseLeftButtonDown(object sender, MouseButtonEventArgs e)
        {
            inMotion = true;

            // start a new motion for moving the camera 
            environment.StartMotion(new CustomMotion(CustomMotionFunction));

            mousePos = e.GetPosition(this);
        }

        private void environment_MouseLeftButtonUp(object sender, MouseButtonEventArgs e)
        {
            inMotion = false;
        }

        private void environment_MouseMove(object sender, MouseEventArgs e)
        {
            if (!inMotion)
                return;

            Point newPos = e.GetPosition(this);
            azimuth -= (float)(mousePos.X - newPos.X) / 100.0f;
            elevation = (float)Math.Max(Math.Min(elevation - (mousePos.Y - newPos.Y) / 100.0f, Math.PI / 3), -Math.PI / 16);
            mousePos = newPos;

            RefreshCamera();
        }

        /// <summary>
        /// Defines the condition for running the custom motion for rendering when camera is being rotated
        /// </summary>
        /// <param name="time">Elapsed time in seconds for the rendered frame</param>
        /// <returns>True if motion should proceed, false otherwise</returns>
        private bool CustomMotionFunction(float time)
        {
            return inMotion;
        }

        /// <summary>
        /// Sets the camera position around the center of the coordinate system according to the elevation and azimuth parameters
        /// </summary>
        private void RefreshCamera()
        {
            // Set coordinates to camera from azimuth and elevation
            Vector3 camPos = new Vector3();
            camPos.FromSphericalCoords(azimuth, elevation, 50);
            environment.Camera.Position = camPos;

            // translate the skybox to the camera's position so we're always in its center
            skyBox.TransformationReference.LocalPosition = environment.Camera.Position;
        }
    }
}
